## ####################################################
## This code requires the R packages:
##   geoR: to draw scatterplots
##   fields: to obtain color scheme (from tim.colors())
## ####################################################


num.time.steps <- 4


## ###########################################
## READ IN THE KERNEL TYPES THAT WE EVENTUALLY MIGHT USE.

source("../../../simul_series/kernel_func.r")

## We use an isotropic version of the GAUSSIAN with an eff.range of 700.
eff.range <- 700
eff.radius <- find.radius.given.eff.range(eff.range, kernel.type="X")
chosen.kernel.df <- data.frame(kernel.type="X", a=eff.radius, b=eff.radius, theta=0.0)
num.kernels <- 1
## ###########################################


## ###########################################
## READ IN OBSERVATION POINTS AND GRID POINTS, DETERMINE BOUNDARIES.

## Read in observations.
obs.pts <- read.csv("../../region_setup/obs_locations.csv")
num.obs <- nrow(obs.pts)

## Read in grid points.
grid.pts <- read.csv("../../region_setup/grid_locations.csv")
## Begin temp stuff.
## cut.uniq <- unique(grid.pts[,1])[seq(1, length(unique(grid.pts[,1])), by=4)]
## grid.pts <- as.data.frame(expand.grid(cut.uniq, cut.uniq))
## names(grid.pts) <- c("x", "y")
## End temp stuff.
num.grid.pts <- nrow(grid.pts)

## Save unique list of delineation points in both directions for grid.
grid.uniq.x <- unique(grid.pts[,1])
grid.uniq.y <- unique(grid.pts[,2])

## Get grid spacing from prediction locatiosn (assume they're all regular).
grid.space <- sort(grid.uniq.x)[2] - sort(grid.uniq.x)[1]

## We know that the range in x and y directions is the range of the
## grid points, with a buffer of half the grid spacing on each side.
x.rng <- c( min(grid.pts[,1])-(0.5*grid.space), max(grid.pts[,1])+(0.5*grid.space) )
y.rng <- c( min(grid.pts[,2])-(0.5*grid.space), max(grid.pts[,2])+(0.5*grid.space) )
## ###########################################


## ###########################################
## READ IN THETAS AND THETA LATTICE.

theta.lattice <- matrix(scan("../theta_lattice.dat"), ncol=2, byrow=T)
theta <- matrix(scan("../theta.dat"), ncol=num.time.steps, byrow=T)
## ###########################################


## #######################################
## FORM K MATRIX FOR COMBINED GRID AND OBS POINTS.

## Make a matrix of combined observation and grid points.
combined.pts <- as.matrix(rbind(obs.pts, grid.pts))

## For each point i at which the process will be simulated, center the
## kernel at point i, and evaluate its height at each of the white
## noise locations.  Put this information in row i of the K matrix.
## a is length of major axis (assumed to be the horizontal)
## b is length of minor axis (assumed to be the vertical)
## Form a weights matrix for each of the kernels.  This matrix is
## weighted to try to correct for the variance.
K.reweight <- matrix(NA, nrow=c(nrow(combined.pts), ncol=nrow(theta.lattice)))

## For each point j at which the process will be simulated, center
## the kernel at point j, and evaluate its corrected height at each
## of the white noise locations. 
K <- matrix(0.0, nrow=nrow(combined.pts), ncol=nrow(theta.lattice))
{
  if (chosen.kernel.df$kernel.type!="X"){
    for (j in 1:nrow(combined.pts)){
      td <- find.standardized.distance(x=theta.lattice[,1], y=theta.lattice[,2], ctr.pt=combined.pts[j,], chosen.kernel.df$a, chosen.kernel.df$b, chosen.kernel.df$theta)
      K[j,] <- find.kernel.height.given.radius.and.transformed.distance(td=td, r=sqrt(chosen.kernel.df$a*chosen.kernel.df$b), kernel.type=chosen.kernel.df$kernel.type)
    }
  }
  
  else{
    ## Find all transformed distances first.
    td <- matrix(NA, nrow=nrow(combined.pts), ncol=nrow(theta.lattice))
    for (j in 1:nrow(combined.pts))
      td[j,] <- find.standardized.distance(x=theta.lattice[,1], y=theta.lattice[,2], ctr.pt=combined.pts[j,], chosen.kernel.df$a, chosen.kernel.df$b, chosen.kernel.df$theta)

    ## Then, take the maximium distance and pass it into the function.
    K <- find.kernel.height.given.radius.and.transformed.distance(td=td, r=sqrt(chosen.kernel.df$a*chosen.kernel.df$b), kernel.type=chosen.kernel.df$kernel.type)
    ##K[K<0] <- 0.0
  }
}
##rm(td, x.rng, j)


## Find the variance stability constant for each row of K.
adj.factor <- apply(K, 1, find.var.stabil.constant, radius=sqrt(chosen.kernel.df$a*chosen.kernel.df$b), kernel.type=chosen.kernel.df$kernel.type)
K.reweight <- K * adj.factor

rm(K, adj.factor)

gc()
## #######################################


## #######################################
## USE WHITE NOISE AND KERNEL HEIGHTS MATRIX TO MAKE Z.  ALSO MAKE A
## VERSION OF Z WITH NUGGET EFFECT.

## Multiply the white noise by kernel heights, and add overall mean,
## to get surface.  (Could change this to use a trend surface instead
## of a constant mean.)
overall.mu <- 50.0

## Get a new field for each day (each column) by multiplying kernel
## weights (K) by theta (each column for a different simulation).
z <- overall.mu + (K.reweight %*% theta)

## Separate into portion of z's which go with observations and portion
## which go with grid points.
z.obs <- round(z[1:num.obs,], 2)
z.grid <- round(z[(num.obs+1):(num.obs+num.grid.pts),], 2)

## Also make version of z.obs with nugget.
set.seed(943817)
nugget.sd <- 3.0
z.obs.w.nugget <- round( z[1:num.obs,] + matrix( rnorm(num.obs*num.time.steps, mean=0.0, sd=nugget.sd), ncol=num.time.steps ), 2 )

rm(overall.mu, z)
## #######################################


## #######################################
## FIND COORDINATES OF ELLIPSES TO REPRESENT SMOOTHING KERNELS.

## First, figure out how many angles at which to evaluate the circle.
num.angles <- 41
phi <- seq(0, 2*pi, length=num.angles)


## Given theta, find r (polar coordinates).
find.r.given.angle <- function(a, b, ellps.rot.angle, theta.for.eval){
  angle.sum <- theta.for.eval - ellps.rot.angle
  r <- (a*b)/sqrt( (a*sin(angle.sum))^2 + (b*cos(angle.sum))^2 )
  return(r)
}
ellps.radii <- find.r.given.angle(chosen.kernel.df$a, chosen.kernel.df$b, chosen.kernel.df$theta, phi)
## #######################################


## #######################################
## MAKE IMAGE OF THE DENSE FIELD FOR SIMULATION.

## Set colors to use in the plots.
library("fields")  # Loading this package for its tim.colors() function.
col.rng <- tim.colors(20)
detach("package:fields")

## Load code for color bar legend.
source("~/R_examples/my_image_plot.r")

## Use code to draw scatterplots with geoR-based functions.
source("~/R_examples/draw_color_scatterplot_w_geoR.r")

## Load code to help make plots with certain ratios.
source("~/R_examples/initialize_aspect_plot.r")

## Make plot with the very densely spaced points.
z.rng <- range(as.vector(rbind(z.grid, z.obs, z.obs.w.nugget)))


## Write out images.
for (t in 1:num.time.steps){

  ## Set the name for the figure and its main title.
  fig.name <- paste("expon_isotropy_time", t, ".pdf", sep="")

  ## Open PDF for writing out the plot.
  initialize.aspect.plot(x.rng, y.rng, file=fig.name, fig.width=4.0,
                         marg=c(0.05, 0.05, 0.05, 0.45),
                         xlab="", ylab="", xaxt="n", yaxt="n",
                         xaxs="i", yaxs="i",
                         cex.main=0.8, cex.axis=0.8, dev="pdf")

  ## Plot image.
  image(grid.uniq.x, grid.uniq.y,
        matrix(z.grid[,t], ncol=length(grid.uniq.y)), zlim=z.rng,
        col=col.rng, add=T)

  ## Add image legend.
  image.plot(legend.only=T, legend.shrink=0.9, zlim=z.rng, offset=0.01,
             col=col.rng)

  dev.off()
  rm(fig.name)
}
rm(t)



## Write out images with smoothing kernels superimposed.
## Make our own grid of points to plot kernels for.
kernel.plot.grid.x <- grid.uniq.x[seq(3, length(grid.uniq.x), by=4)]
kernel.plot.grid.y <- grid.uniq.y[seq(3, length(grid.uniq.y), by=4)]
kernel.plot.grid <- expand.grid(kernel.plot.grid.x, kernel.plot.grid.y)
rm(kernel.plot.grid.x, kernel.plot.grid.y)
for (t in 1:num.time.steps){

  ## Set the name for the figure and its main title.
  fig.name <- paste("expon_isotropy_w_kernels_time", t, ".pdf", sep="")

  ## Open PDF for writing out the plot.
  initialize.aspect.plot(x.rng, y.rng, file=fig.name, fig.width=4.0,
                         marg=c(0.05, 0.05, 0.05, 0.45),
                         xlab="", ylab="", xaxt="n", yaxt="n",
                         xaxs="i", yaxs="i",
                         cex.main=0.8, cex.axis=0.8, dev="pdf")

  ## Plot image.
  image(grid.uniq.x, grid.uniq.y,
        matrix(z.grid[,t], ncol=length(grid.uniq.y)), zlim=z.rng,
        col=col.rng, add=T)

  ## Add kernels (need to be shrunk by a large factor).
  shrink.factor <- 25.0
  for (i in 1:nrow(kernel.plot.grid))
    lines(kernel.plot.grid[i,1]+(ellps.radii*cos(phi)/shrink.factor), kernel.plot.grid[i,2]+(ellps.radii*sin(phi)/shrink.factor))
  
  ## Add image legend.
  image.plot(legend.only=T, legend.shrink=0.9, zlim=z.rng, offset=0.01,
             col=col.rng)

  dev.off()
  rm(fig.name, shrink.factor, i)
}
rm(t)
## #######################################


## #######################################
## DRAW AN IMAGE OF THE KERNEL HEIGHTS.

## Set the name for the figure and its main title.
fig.name <- "expon_isotropic_kernel_hts.pdf"

## Open PDF for writing out the plot.
initialize.aspect.plot(x.rng, y.rng, file=fig.name, fig.width=4.1,
                       marg=c(0.05, 0.05, 0.05, 0.55),
                       xlab="", ylab="", xaxt="n", yaxt="n",
                       xaxs="i", yaxs="i", bty="n", dev="pdf")

## We wil draw the kernel so that it is centered in the center of our region.
kernel.ctr.pt <- apply(grid.pts, 2, mean)

## Evaluate the smoothing kernel at all grid points.
td <- find.standardized.distance(x=grid.pts[,1], y=grid.pts[,2], ctr.pt=kernel.ctr.pt, a=chosen.kernel.df$a, b=chosen.kernel.df$b, theta=chosen.kernel.df$theta)
hts <- find.kernel.height.given.radius.and.transformed.distance(td=td, r=sqrt(chosen.kernel.df$a*chosen.kernel.df$b), kernel.type=chosen.kernel.df$kernel.type, num.uniq.lattice.y.pts=length(grid.uniq.y))

## Find range of the hts.
hts.z.rng <- c(0.0, 0.006)
##hts.z.rng <- range(hts)

## Plot these heights using image.
image(grid.uniq.x, grid.uniq.y, matrix(hts, ncol=length(grid.uniq.y)), zlim=hts.z.rng, col=col.rng, add=T)

## Add image legend.
image.plot(legend.only=T, legend.shrink=0.9, offset=0.01,
           zlim=hts.z.rng, col=col.rng)

dev.off()
rm(fig.name, kernel.ctr.pt, td, hts, hts.z.rng)
## #######################################


## #######################################
## DRAW A PICTURE OF THE KERNEL (JUST THE OUTLINE).

## Set the name for the figure and its main title.
fig.name <- "expon_isotropic_kernel_outline.pdf"

## Open PDF for writing out the plot.
initialize.aspect.plot(ellps.radii*cos(phi), ellps.radii*sin(phi),
                       file=fig.name, fig.width=4.0,
                       marg=c(0.05, 0.05, 0.05, 0.05),
                       xlab="", ylab="", xaxt="n", yaxt="n",
                       xaxs="i", yaxs="i", bty="n", dev="pdf")

lines(ellps.radii*cos(phi), ellps.radii*sin(phi))

dev.off()
rm(fig.name)
## #######################################
